package com.technotgeek.sketchapp;

/*
 * Sample application constraint
 * Max VM Application heap size should not be less than 64 Mbyte
 */
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.util.HashMap;

import android.app.Activity;
import android.app.AlertDialog;
import android.app.Dialog;
import android.content.ActivityNotFoundException;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.graphics.Bitmap;
import android.graphics.Bitmap.CompressFormat;
import android.graphics.BitmapFactory;
import android.graphics.Color;
import android.hardware.Sensor;
import android.hardware.SensorEvent;
import android.hardware.SensorEventListener;
import android.hardware.SensorManager;
import android.net.Uri;
import android.os.Bundle;
import android.os.Environment;
import android.util.Log;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.RelativeLayout;
import android.widget.Toast;

import com.samsung.samm.common.SObjectStroke;
import com.samsung.spensdk.SCanvasConstants;
import com.samsung.spensdk.SCanvasView;
import com.samsung.spensdk.applistener.HistoryUpdateListener;
import com.samsung.spensdk.applistener.SCanvasInitializeListener;
import com.samsung.spensdk.applistener.SettingStrokeChangeListener;

public class SPen_Example_Simple extends Activity implements SensorEventListener{
	private SensorManager sensorManager;
	private long lastUpdate;

	public static final String TAG = "Example1";
	private final int REQUEST_CODE_SELECT_IMAGE_BACKGROUND = 106;	

	public static final String DEFAULT_APP_IMAGEDATA_DIRECTORY = "SPenSDK2.1/images";

	public static final String EXTRA_IMAGE_PATH = "path";
	public static final String EXTRA_IMAGE_NAME = "filename";

	public static final String SAVED_FILE_EXTENSION = "png";
	private static final int LOAD_BACKGROUND = 100;

	private static final int DIALOG_CHANGE_BACKGROUND = 1;
	private SCanvasView mSCanvas;
	private Button mPenBtn;
	private Button mEraserBtn;
	private Button mUndoBtn;
	private Button mRedoBtn; 
	private File mFolder = null;
	Context mContext = null;

	private int mCheckedItem;
	private Bitmap bmBackgroundBitmap=null;
	private boolean bgPresent=true;

	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);

		setContentView(R.layout.simple_main);

		mContext = this;

		mCheckedItem = 0;
		mPenBtn = (Button) findViewById(R.id.settingBtn);
		mPenBtn.setOnClickListener(mBtnClickListener);
		mPenBtn.setTextColor(Color.WHITE);
		mEraserBtn = (Button) findViewById(R.id.eraseBtn);
		mEraserBtn.setOnClickListener(mBtnClickListener);
		mUndoBtn = (Button) findViewById(R.id.undoBtn);
		mUndoBtn.setOnClickListener(undoNredoBtnClickListener);
		mRedoBtn = (Button) findViewById(R.id.redoBtn);
		mRedoBtn.setOnClickListener(undoNredoBtnClickListener);

		mSCanvas = (SCanvasView) findViewById(R.id.canvas_view);

		// Resource
		HashMap<String,Integer> settingResourceMap = new HashMap<String, Integer>();
		// Layout 
		settingResourceMap.put(SCanvasConstants.LAYOUT_PEN_SPINNER, R.layout.mspinner);
		// Locale
		settingResourceMap.put(SCanvasConstants.LOCALE_PEN_SETTING_TITLE, R.string.pen_settings);
		settingResourceMap.put(SCanvasConstants.LOCALE_ERASER_SETTING_TITLE, R.string.eraser_settings);
		settingResourceMap.put(SCanvasConstants.LOCALE_ERASER_SETTING_CLEARALL, R.string.clear_all);   
		settingResourceMap.put(SCanvasConstants.LOCALE_ERASER_SETTING_CLEARALL_MESSAGE, R.string.confirm_clear_all); 

		// Create Setting View
		boolean bClearAllVisibileInEraserSetting = true;
		RelativeLayout settingViewContainer = (RelativeLayout) findViewById(R.id.canvas_container);		
		mSCanvas.createSettingView(settingViewContainer, settingResourceMap, bClearAllVisibileInEraserSetting);    	  

		mSCanvas.setSCanvasInitializeListener(mSCanvasInitializeListener);
		mSCanvas.setHistoryUpdateListener(mHistoryUpdateListener);    	
		mSCanvas.setSettingStrokeChangeListener(mSettingStrokeChangeListener);


		// create basic save/road file path
		File sdcard_path = Environment.getExternalStorageDirectory();
		mFolder =  new File(sdcard_path, DEFAULT_APP_IMAGEDATA_DIRECTORY);
		if(!mFolder.exists()){
			if(mFolder.mkdirs()==false){
				Log.e(TAG, "Default Save Path Creation Error");
				return ;
			}
		}

		mUndoBtn.setEnabled(false);
		mRedoBtn.setEnabled(false);
		sensorManager = (SensorManager) getSystemService(SENSOR_SERVICE);
		sensorManager.registerListener(this,
				sensorManager.getDefaultSensor(Sensor.TYPE_ACCELEROMETER),
				SensorManager.SENSOR_DELAY_NORMAL);
		lastUpdate = System.currentTimeMillis();

	}
	//==================================================
		// Call Gallery
		//==================================================
		private void callGalleryForInputImage(int nRequestCode) {
			try {
				Intent galleryIntent;
				galleryIntent = new Intent(); 
				galleryIntent.setAction(Intent.ACTION_GET_CONTENT);				
				galleryIntent.setType("image/*");
				galleryIntent.setClassName("com.cooliris.media", "com.cooliris.media.Gallery");
				startActivityForResult(galleryIntent, nRequestCode);
			} catch(ActivityNotFoundException e) {
				Intent galleryIntent;
				galleryIntent = new Intent();
				galleryIntent.setAction(Intent.ACTION_GET_CONTENT);
				galleryIntent.setType("image/*");
				startActivityForResult(galleryIntent, nRequestCode);
				e.printStackTrace();
			}		
		}

	@Override
	protected void onActivityResult(int requestCode, int resultCode, Intent data) {
		super.onActivityResult(requestCode, resultCode, data);

		if (resultCode == RESULT_OK) {
			
			if(requestCode == REQUEST_CODE_SELECT_IMAGE_BACKGROUND) {    			
				Uri imageFileUri = data.getData();
				String strBackgroundImagePath = AnimationUtils.getRealPathFromURI(this, imageFileUri);

				// Check Valid Image File
				if(!AnimationUtils.isValidImagePath(strBackgroundImagePath))
				{
					Toast.makeText(this, "Invalid image path or web image", Toast.LENGTH_LONG).show();	
					return;
				}    			

				// Set background image on the image view
				bmBackgroundBitmap = BitmapFactory.decodeFile(strBackgroundImagePath);
				int nWidth = mSCanvas.getWidth();
				int nHeight = mSCanvas.getHeight();
				bmBackgroundBitmap = Bitmap.createScaledBitmap(bmBackgroundBitmap, nWidth, nHeight, true);
				if(bmBackgroundBitmap!=null){    			
				 mSCanvas.setBGImage(bmBackgroundBitmap); 				
				}
			else
				Toast.makeText(mContext, "Fail to set Background Image Path.", Toast.LENGTH_LONG).show();    			
		//mSCanvas.setBGImagePath(strBackgroundImagePath);
			}			
		}
	}

	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		getMenuInflater().inflate(R.menu.example_simple_menu, menu);
		return super.onCreateOptionsMenu(menu);

	}

	@Override
	public boolean onOptionsItemSelected(MenuItem item) {
		super.onOptionsItemSelected(item);

		boolean bSaveOnlyForeground = true;
		switch (item.getItemId()) {
		case R.id.options_save_fg:
			Log.i(TAG, "SAVE Only Foreground Image");
			bSaveOnlyForeground = true;
			if (saveCanvasImage(bSaveOnlyForeground))
				Log.d(TAG, "SAVE succeed");
			else
				Log.d(TAG, "SAVE failed");
			return true;
		case R.id.clearbg:
			Log.i(TAG, "clear background");
			initBackground();
			bgPresent=false;
			return true;
		case R.id.restorebg:
			Log.i(TAG, "restore background");
			if(bmBackgroundBitmap!=null){    			
				 mSCanvas.setBGImage(bmBackgroundBitmap); 				
			bgPresent=true;
			}
			else
				Toast.makeText(mContext, "Fail to set Background Image Path.", Toast.LENGTH_LONG).show();    			
			return true;
		
		case R.id.selectImage:
			Log.i(TAG, "Select Image");
			callGalleryForInputImage(REQUEST_CODE_SELECT_IMAGE_BACKGROUND);
			return true;       	
		}

		return false;
	}

	public boolean saveCanvasImage(boolean bSaveOnlyForegroundImage) {    	
		Bitmap bmCanvas = mSCanvas.getCanvasBitmap(bSaveOnlyForegroundImage);

		if (!(mFolder.exists()))
			if(!mFolder.mkdirs())
				return false;
		String savePath = mFolder.getPath() + '/' + ExampleUtils.getUniqueFilename(mFolder, "image", SAVED_FILE_EXTENSION);
		Log.d(TAG, "Save Path = " + savePath);

		return saveBitmapPNG(savePath, bmCanvas);
	}

	boolean saveBitmapPNG(String strFileName, Bitmap bitmap){		
		if(strFileName==null || bitmap==null)
			return false;

		boolean bSuccess1 = false;	
		boolean bSuccess2;
		boolean bSuccess3;
		File saveFile = new File(strFileName);			

		if(saveFile.exists()) {
			if(!saveFile.delete())
				return false;
		}

		try {
			bSuccess1 = saveFile.createNewFile();
		} catch (IOException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		}

		OutputStream out = null;
		try {
			out = new FileOutputStream(saveFile);
			bSuccess2 = bitmap.compress(CompressFormat.PNG, 100, out);			
		} catch (Exception e) {
			e.printStackTrace();			
			bSuccess2 = false;
		}
		try {
			if(out!=null)
			{
				out.flush();
				out.close();
				bSuccess3 = true;
			}
			else
				bSuccess3 = false;

		} catch (IOException e) {
			e.printStackTrace();
			bSuccess3 = false;
		}finally
		{
			if(out != null)
			{
				try {
					out.close();
				} catch (IOException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}				
			}			
		}		

		return (bSuccess1 && bSuccess2 && bSuccess3);
	}	

	public boolean loadCanvas(String fileName) {

		String loadPath = mFolder.getPath() + '/' + fileName;
		
		Log.i(TAG, "Load Path = " + loadPath);
		
		Bitmap bmForeground = BitmapFactory.decodeFile(loadPath);
		int nWidth = mSCanvas.getWidth();
		int nHeight = mSCanvas.getHeight();
		bmForeground = Bitmap.createScaledBitmap(bmForeground, nWidth, nHeight, true);
		mSCanvas.setClearImageBitmap(bmForeground);
	
		return true;
	}
	private String[] mBgNames = { "background 1", "background 2", "background 3", "clear background", };

	protected Dialog onCreateDialog(int id) {
		switch (id) {
		case DIALOG_CHANGE_BACKGROUND:
			AlertDialog.Builder builder = new AlertDialog.Builder(mContext);
			builder.setSingleChoiceItems(mBgNames, mCheckedItem, new DialogInterface.OnClickListener() {

				public void onClick(DialogInterface dialog, int which) {                  
					mCheckedItem = which;
					Bitmap bm = null;
					switch(which) {
					case 0:
						bm = BitmapFactory.decodeResource(mContext.getResources(), R.drawable.letter_bg_grass);
						break;
					case 1:
						bm = BitmapFactory.decodeResource(mContext.getResources(), R.drawable.letter_bg_present);
						break;
					case 2:
						bm = BitmapFactory.decodeResource(mContext.getResources(), R.drawable.letter_bg);
						break;
					case 3:
						initBackground();
						break;
					}	
					if(bm != null) {
						mSCanvas.setBGImage(bm);
					}
					removeDialog(DIALOG_CHANGE_BACKGROUND);
				}
			});
			return builder.create();
		}
		return super.onCreateDialog(id);
	}

	private void initBackground() {
		mSCanvas.setBGColor(0xffffffff);
	}

	

	private OnClickListener undoNredoBtnClickListener = new OnClickListener() {
		
		public void onClick(View v) {
			if (v.equals(mUndoBtn)) {
				mSCanvas.undo();
			} else if (v.equals(mRedoBtn)) {
				mSCanvas.redo();
			}

			mUndoBtn.setEnabled(mSCanvas.isUndoable());
			mRedoBtn.setEnabled(mSCanvas.isRedoable());
		}
	};

	OnClickListener mBtnClickListener = new OnClickListener() {

		
		public void onClick(View v) {
			int nBtnID = v.getId();
			// If the mode is not changed, open the setting view. If the mode is same, close the setting view. 
			if(nBtnID == mPenBtn.getId()){				
				if(mSCanvas.getCanvasMode()==SCanvasConstants.SCANVAS_MODE_INPUT_PEN){
					mSCanvas.toggleShowSettingView(SCanvasConstants.SCANVAS_SETTINGVIEW_PEN);
				}
				else{
					mSCanvas.setCanvasMode(SCanvasConstants.SCANVAS_MODE_INPUT_PEN);
					mSCanvas.showSettingView(SCanvasConstants.SCANVAS_SETTINGVIEW_PEN, false);					
					updateModeState();
				}
			}
			else if(nBtnID == mEraserBtn.getId()){
				if(mSCanvas.getCanvasMode()==SCanvasConstants.SCANVAS_MODE_INPUT_ERASER){
					mSCanvas.toggleShowSettingView(SCanvasConstants.SCANVAS_SETTINGVIEW_ERASER);
				}
				else {
					mSCanvas.setCanvasMode(SCanvasConstants.SCANVAS_MODE_INPUT_ERASER);
					mSCanvas.showSettingView(SCanvasConstants.SCANVAS_SETTINGVIEW_ERASER, false);
					updateModeState();
				}
			}
		}
	};

	private void updateModeState(){
		int nCurMode = mSCanvas.getCanvasMode();
		if(nCurMode==SCanvasConstants.SCANVAS_MODE_INPUT_PEN) {
			mPenBtn.setTextColor(Color.WHITE);
			mEraserBtn.setTextColor(Color.BLACK);
		}
		else if(nCurMode==SCanvasConstants.SCANVAS_MODE_INPUT_ERASER) {
			mPenBtn.setTextColor(Color.BLACK);
			mEraserBtn.setTextColor(Color.WHITE);
		}
	}

	//------------------------------------------------
	// SettingView Listener 
	//------------------------------------------------				
	SettingStrokeChangeListener mSettingStrokeChangeListener = new SettingStrokeChangeListener() {

		
		public void onClearAll(boolean bClearAllCompleted) {
			if(bClearAllCompleted) Log.i(TAG, "Clear All is completed");
		}

		
		public void onEraserWidthChanged(int eraserWidth) {				
			Log.i(TAG, "Eraser width is changed : " + eraserWidth);				
		}

		
		public void onStrokeColorChanged(int strokeColor) {				
			Log.i(TAG, "Pen Color is changed : " + strokeColor);				
		}

		
		public void onStrokeStyleChanged(int strokeStyle) {

			if (strokeStyle == SObjectStroke.SAMM_STROKE_STYLE_PENCIL)
				Log.i(TAG, "Stroke Style = Pen");
			else if (strokeStyle == SObjectStroke.SAMM_STROKE_STYLE_BRUSH)
				Log.i(TAG, "Stroke Style = Brush");
			else if (strokeStyle == SObjectStroke.SAMM_STROKE_STYLE_CRAYON)
				Log.i(TAG, "Stroke Style = Pencil Crayon");    		
			else if (strokeStyle == SObjectStroke.SAMM_STROKE_STYLE_MARKER)
				Log.i(TAG, "Stroke Style = Marker");	

		}

		
		public void onStrokeWidthChanged(int strokeWidth) {					
			Log.i(TAG, "Pen width is changed : " + strokeWidth);				
		}

		
		public void onStrokeAlphaChanged(int strokeAlpha) {					
			Log.i(TAG, "Pen alpha is changed : " + strokeAlpha);				
		}

	};

	private HistoryUpdateListener mHistoryUpdateListener = new HistoryUpdateListener() {

		
		public void onHistoryChanged(boolean bUndoable, boolean bRedoable) {

			mUndoBtn.setEnabled(bUndoable);
			mRedoBtn.setEnabled(bRedoable);

		}
	};


	SCanvasInitializeListener mSCanvasInitializeListener = new SCanvasInitializeListener() {
		
		public void onInitialized() {
			Bitmap bg = BitmapFactory.decodeResource(getResources(), R.drawable.letter_bg_grass);
			if(bg == null)
				return;
			mSCanvas.setBackgroundImage(bg);
			bg.recycle();
		}
	};

	public void onAccuracyChanged(Sensor sensor, int accuracy) {
		// TODO Auto-generated method stub
		
	}
	public void onSensorChanged(SensorEvent event) {
		if (event.sensor.getType() == Sensor.TYPE_ACCELEROMETER) {
			getAccelerometer(event);
		}
	}
	private void getAccelerometer(SensorEvent event) {
		float[] values = event.values;
		// Movement
		float x = values[0];
		float y = values[1];
		float z = values[2];

		float accelationSquareRoot = (x * x + y * y + z * z)
				/ (SensorManager.GRAVITY_EARTH * SensorManager.GRAVITY_EARTH);
		long actualTime = System.currentTimeMillis();
		if (accelationSquareRoot >= 2) //
		{
			if (actualTime - lastUpdate < 200) {
				return;
			}
			lastUpdate = actualTime;
			Toast.makeText(this, "Device was shuffled", Toast.LENGTH_SHORT)
					.show();
			if(bgPresent){
				initBackground();
				bgPresent=false;
			}
			else{
				if(bmBackgroundBitmap!=null){    			
					 mSCanvas.setBGImage(bmBackgroundBitmap); 				
				bgPresent=true;
				}
			}
				
				
		}
	}
	@Override
	protected void onResume() {
		super.onResume();
		// register this class as a listener for the orientation and
		// accelerometer sensors
		sensorManager.registerListener(this,
				sensorManager.getDefaultSensor(Sensor.TYPE_ACCELEROMETER),
				SensorManager.SENSOR_DELAY_NORMAL);
	}

	@Override
	protected void onPause() {
		// unregister listener
		super.onPause();
		sensorManager.unregisterListener(this);
	}
}
